function [value,shuffle] = shuffle_transform(image,im2,radius, transform_type)
% SHUFFLE_TRANSFORM: A shuffle transform based on an implementation by Steve Marsland
%
%    Licensing
%
%      -
%
%    GENERAL
%
%      Function
%
%    INPUT/S
%
%      -image: first image
%      -im2: second image
%      -radius: the shuffle radius
%      -transform_type: 'sum', 'median' or other (proper shuffle as default, where best match only counts)

%    OUTPUT/S
%
%      -Omitted
%
%    PENDING WORK
%
%      -
%
%    KNOWN BUG/S
%
%      -None.
%
%    COMMENT/S
%
%      -
%
%    RELATED FUNCTION/S
%
%      LOADTAGGEDSQUENCE
%
%    ABOUT
%
%      -Created:     November 2010
%      -Last update: November 21st, 2010
%      -Revision:    0.0.1
%      -Author:      Dr. R. S. Schestowitz
% ==============================================================

use_edges=1; % whether or not edge detection should be considered a weighted factor in evaluation
use_gradient=1; % whether or not image gradient should be considered a weighted factor in evaluation
simple_difference=1; % overriding the above option, this just takes the simple point-to-point difference between images
sigma=1; % for detection of edges/gradients, the value of sigma needs to be specified
weight_factor=0.5;
	   % how to scale intensity vs. gradient or edge. Use 0 for edge only, 1 for intensity only

if (simple_difference)
	% size(im2)
	shuffle=abs(image-im2);
	value = weight_factor * mean(shuffle(:));
else % perform shuffle calculations, optionally with some gradient and edge data factored in
	shuffle = zeros(size(image));
	shuffle_edge = zeros(size(image));
	shuffle_gradient = zeros(size(image));
	
	for i=radius+1:size(image,1)-radius
    	for j=radius+1:size(image,2)-radius
        	template = im2(i-radius:i+radius,j-radius:j+radius);
        	diff = abs(template-image(i,j)); 
        	if strcmp(transform_type,'sum') % summing difference
        	  shuffle(i,j) = sum(diff(:));
        	else if strcmp(transform_type,'median') % median rather than sum or average	
        		shuffle(i,j) = median(diff(:));
  	             else % proper shuffle, taking just best fitting pixel into account
        		shuffle(i,j) = min(diff(:));
                     end	
        	end
        	
    	  end
	end
	
	%% use gradient (experimental)
	
	if (use_gradient)
		[imageX,imageY]=gaussgradient(image, sigma);
		image=imageX/2+imageY/2;
		[im2X,im2Y]=gaussgradient(im2, sigma);
		im2=im2X/2+im2Y/2;
		
		% figure;
		% imshow(im2);
		
		for i=radius+1:size(image,1)-radius
    		  for j=radius+1:size(image,2)-radius
        		template = im2(i-radius:i+radius,j-radius:j+radius);
        		diff = abs(template-image(i,j));
        		if strcmp(transform_type,'sum') % summing difference
                		shuffle_gradient(i,j) = sum(diff(:));
        		else if strcmp(transform_type,'median') % median rather than sum or average
                		shuffle_gradient(i,j) = median(diff(:));
                		else % proper shuffle, taking just best fitting pixel into account
                        		shuffle_gradient(i,j) = min(diff(:));
                		end
        		end
    		  end
		end
	end
	
	%% now deal with edge detectors
	
	if (use_edges)
		image=edge(image,'sobel',[],'both','nothinning');
		im2=edge(im2,'sobel',[],'both','nothinning');
		
		% figure;
		% imshow(im2);
		
		for i=radius+1:size(image,1)-radius
    		  for j=radius+1:size(image,2)-radius
        		template = im2(i-radius:i+radius,j-radius:j+radius);
        		diff = abs(template-image(i,j));
        		if strcmp(transform_type,'sum') % summing difference
                		shuffle_edge(i,j) = sum(diff(:));
        		else if strcmp(transform_type,'median') % median rather than sum or average
                		shuffle_edge(i,j) = median(diff(:));
                		else % proper shuffle, taking just best fitting pixel into account
                        		shuffle_edge(i,j) = min(diff(:));
                		end
        		end
    		  end
		end
	end
	
	value = weight_factor * mean(shuffle(:)) + (1 - weight_factor) * mean(shuffle_edge(:)) + (1 - weight_factor) * mean(shuffle_gradient(:)); % get a figure representative of the region
end
