function [new_image] = loadtaggedsequence(directory, wildcard, threshold, transform_type, frame_size, shuffle_radius, draw_grid, arrow_type, verbosity, output_type, boundary_type, using_octave)
% LOADTAGGEDSQUENCE: Open and test tagged images of the heart
%
%    Licensing
%
%      GNU GPL version 3
%
%    GENERAL
%
%      Script/function
%
%    INPUT/S
%
%      -directory: location of images
%      -wildcard: filetype/pattern to find in "directory"
%	 -threshold: colour of tags/markers (landmark points) based on contrast of image
%      -transform_type: 'sum', 'median' or other (proper shuffle as default, where best match only counts)
%      -frame_size: the size of the image frame considered when measuring shuffle distances
%      -shuffle_radius: the size of the shuffle window under consideration
%      -draw_grid: boolean for display of manual grid/points (drawing or not drawing)
%      -arrow_type: 'gradient', 'black', 'thick gradient', or 'thick black', 'dot' or 'none'
%	  -verbosity: whether or not information of limited importance should be echoed in the command lie
%      -output_type: 'surface', 'surf', 'mesh', or 'image' (anything else -- with default as 'none' -- will yield no visual output)
%	   -boundary_type: 'matrix', 'circle', 'list', 'random', or 'manual'
%	   -using_octave: boolean - whether Octave is used or MATLAB (to resolve incompatibilities)
%       
%
%    OUTPUT/S
%
%      -Omitted
%
%    PENDING WORK
%
%      -
%
%    KNOWN BUG/S
%
%      -None.
%
%    COMMENT/S
%
%      -This is a root file which serves as patron to key processing
%
%    RELATED FUNCTION/S
%
%      SHUFFLE_TRANSFORM FIESTA_SIMULATION
%
%    ABOUT
%
%      -Created:     November 2010
%      -Last update: November 10th, 2010
%      -Revision:    0.0.1
%      -Author:      Dr. R. S. Schestowitz
% ==============================================================
%
% ╔══════════════════════════════════════════════╗
% ║ loadtaggedsequence ║ Dr. R. S. Schestowitz   ║
% ╟──────────────────────────────────────────────╢
% ║ Experimental options                         ║
% ╚══════════════════════════════════════════════╝

draw_circles=1; % to be implemented, based on biosig library implementation - NOTE: implemented in Dec. 2010
triangulate=0; % triangulation or contouring, enabled by default and just connects dots
save_images=1;
play_video=1;
output_paramaters_at_end=1;

save_directory=strcat('~/Main/IT/Video_demos/experiments_with_results/shuffle-',num2str(frame_size),'-',num2str(shuffle_radius), '/');
% for CS:
% save_directory=strcat('/home/S00/schestr0/experiments_with_results/shuffle-',num2str(frame_size),'-',num2str(shuffle_radius), '/');

if (save_images)
   system(strcat('mkdir ',' ',save_directory)); % MATLAB chomps trailing spaces
end

rotation_angle=0;

% ╔══════════════════════════════════════════════╗
% ║ loadtaggedsequence ║ Dr. R. S. Schestowitz   ║
% ╟──────────────────────────────────────────────╢
% ║ Rotation detection options                   ║
% ╚══════════════════════════════════════════════╝

measure_tangent_component=1; % whether or not the twist of the heart is being measured
clockwise=0; % accumulator for direction of rotation
move_direction='none';

% ╔══════════════════════════════════════════════╗
% ║ loadtaggedsequence ║ Dr. R. S. Schestowitz   ║
% ╟──────────────────────────────────────────────╢
% ║ Inflation detection options                  ║
% ╚══════════════════════════════════════════════╝

measure_inflation_component=1; % whether or not the heart's expansion should be measured for plotting
inflation=0; % accumulator for inflation/deflation
move_direction='none';

% ╔══════════════════════════════════════════════╗
% ║ loadtaggedsequence ║ Dr. R. S. Schestowitz   ║
% ╟──────────────────────────────────────────────╢
% ║ Options relating to gradient                 ║
% ╚══════════════════════════════════════════════╝

search_along_gradient=0; % first iteration only, for initialisation
search_along_gradient_every_iteration=0;
gradient_search_range=2;

boundaries_line_width=1; % must be scalar
boundaries_line_type='-.'; % {-} | -- | : | -. | none
boundaries_line_marker='.';
boundaries_line_colour='g';

cross_line_colour='r';

% ╔══════════════════════════════════════════════╗
% ║ loadtaggedsequence ║ Dr. R. S. Schestowitz   ║
% ╟──────────────────────────────────────────────╢
% ║ Grid parameters and shuffle                  ║
% ╚══════════════════════════════════════════════╝

%for grid/matrix of points:

x_start = 150;
y_start = 135;
step_x = 5;
step_y = 5;
x_max = 200;
y_max = 170;

%for circle/manual placement of points:

radius=19;
round_x0=181;
round_y0=169;
round_skip_rate=3;

%for manual placement of points:

n_manual_input_points=5;


% What remains are some commonly used values:


% // Fiesta dataset:
%
% boundary_type='matrix';
%
% x_start = 150;
% y_start = 135;
% step_x = 5;
% step_y = 5;
% x_max = 200;
% y_max = 170;
% 
% // Small resolution from Zhen's Ph.D thesis:
%
% boundary_type='matrix';
% 
% x_start = 18;
% y_start = 18;
% step_x = 10;
% step_y = 10;
% x_max = 70;
% y_max = 70;

%boundary_type='circle';  % circle is intended to estimate a fit
                                          %% to the heart's walls, but it requires 
                                          %% coordinates and radius for initialisation
%
%radius=13;
%round_x0=170;
%round_y0=125;
%round_skip_rate=5;



%boundary_type='circle';   %for synthetic/flat
%
%radius=150;
%round_x0=495;
%round_y0=600;
%round_skip_rate=5;

% shuffle parameters - to override input parameters




%boundary_type='manual';   
%
%radius=0;
%round_x0=0;
%round_y0=0;
%
%Xarray=0;
%Yarray=0;
%
%n_manual_input_points=12;
%




% shuffle parameters - to override input parameters of this function
%
% frame_size=5;
% shuffle_radius=2;
% transform_type='min';

% frame_size=9;
% shuffle_radius=5;
% transform_type='sum';

% frame_size=9;
% shuffle_radius=5;
% transform_type=median';

% ╔══════════════════════════════════════════════╗
% ║ loadtaggedsequence ║ Dr. R. S. Schestowitz   ║
% ╟──────────────────────────────────────────────╢
% ║ Presets/initialisation                       ║
% ╚══════════════════════════════════════════════╝
% 
% initialisation/hacks - may need cleaning up

first_image=1;
old_boundaries=[[0,0];[0,0]];

% arrow colour schemes - set up colours, tweak with caution

if (strcmp(arrow_type,'gradient'))
	arrow_color_origin=255;
	arrow_color_second=240;
	arrow_color_third=210;
	arrow_color_fourth=230;
	arrow_color_blur=190;
	thick_arrows=0; % boolean
end
if (strcmp(arrow_type,'black'))
 	arrow_color_origin=0;
 	arrow_color_second=0;
 	arrow_color_third=0;
 	arrow_color_fourth=0;
 	arrow_color_blur=0;
	thick_arrows=0;
end	
if (strcmp(arrow_type,'thick gradient'))
	arrow_color_origin=255;
	arrow_color_second=240;
	arrow_color_third=210;
	arrow_color_fourth=230;
	arrow_color_blur=190;
	thick_arrows=1;
end
if (strcmp(arrow_type,'thick black'))
 	arrow_color_origin=0;
 	arrow_color_second=0;
 	arrow_color_third=0;
 	arrow_color_fourth=0;
 	arrow_color_blur=0;
	thick_arrows=1;
end

if (strcmp(arrow_type,'dot'))
        arrow_color_origin=0;
end


% ╔══════════════════════════════════════════════╗
% ║ loadtaggedsequence ║ Dr. R. S. Schestowitz   ║
% ╟──────────────────────────────────────────────╢
% ║ Body				                          ║
% ╚══════════════════════════════════════════════╝
% 
% body of algorithm starts, so get data first

cd(directory);

if (using_octave)
  files_list=ls(wildcard); % only works correctly in Octave
else
  files_list=dir(wildcard);
end

	% files_list=ls('*.gif');
	% One image: files_list=('seq-tag-1.png');

% opening [print filenames if necessary, also number of images]

n_images=size(files_list,1);
	
if (verbosity==1)
	disp('Opening the following files in order:');
	files_list
	if (strcmp(output_type, 'image') && triangulate) 
        	disp('displaying lines on top for image for purposes of annotation')
        end
end

% main loop

	% img = sopen("~/Main/IT/Programs/MR-Cardiac/i148713.MRDC.1");
for i=1:1:n_images,
	% img_metadata(i) = sopen(files_list(i,:))
	% a=imread(files_list(i,:));
	% imshow(a)
	% size(a)
	% files_list(i,:)
		if (strcmp(wildcard, '--width=1 -X *.jpg') || strcmp(wildcard, '*.bmp') || strcmp(wildcard, '--width=1 -X *.bmp') || (strcmp(wildcard, '--width=1 -X  *.png')))  % some test sets are 3-D, so need hack
			if (using_octave)
				full_data=imread(files_list(i,:));
			else	
		        	full_data=imread(files_list(i,:).name);
			end
			if (abs(rotation_angle)) % if non-zero
				full_data(:,:,1)=imrotate(full_data(:,:,1),rotation_angle,'bilinear');
			end	
			img(:,:,i)=full_data(:,:,1);
			if (first_image==0)    % restore previous image as well
				if (using_octave)	
			   		full_data=imread(files_list(i-1,:));
				else
		           		full_data=imread(files_list(i-1,:).name);
				end
				if (abs(rotation_angle)) % if non-zero
					full_data(:,:,1)=imrotate(full_data(:,:,1),rotation_angle,'bilinear');
				end	
				img(:,:,i-1)=full_data(:,:,1);
			end
		else
			% size(imread(files_list(i,:)))
			if (using_octave)	
				full_data=imread(files_list(i,:));
			else
				full_data=imread(files_list(i,:).name);
			end
			if (abs(rotation_angle)) % if non-zero
				img(:,:,i)=imrotate(full_data,rotation_angle,'bilinear');
			else
				img(:,:,i)=	full_data;
			end	
			if (first_image==0)  
				if (using_octave)
			 		full_data=imread(files_list(i-1,:));
				else
					full_data=imread(files_list(i-1,:).name);
				end
				if (abs(rotation_angle)) % if non-zero
					img(:,:,i-1)=imrotate(full_data,rotation_angle,'bilinear');
				else
					img(:,:,i-1)=full_data;
				end	
			end
		end
		
		if (draw_grid==1)
			for j=x_start:step_x:x_max,
	   	  	img(j,:,i)=0;
			end
			for j=y_start:step_y:y_max,
	   	  	img(:,j,i)=0;
			end
		end
		
		
        	if (first_image)
        		if (strcmp(boundary_type,'matrix'))
				%boundaries=[[x_start,y_start];[x_start,y_start+step_y]; ...
				%[x_start,y_start+step_y*2]; [x_start,y_start]; ...
				%[x_start+step_x,y_start];[x_start+step_x*2,y_start]; ...
				%[x_start+step_x*2,y_start+step_y];[x_start+step_x*2,y_start+step_y*2]; ...
				%[x_start+step_x,y_start+step_y];[x_start+step_x,y_start+step_y*2];...
				%
				%[x_start,y_start+step_y*3];[x_start+step_x*3,y_start]; ...
				%[x_start+step_x*3,y_start+step_y];[x_start+step_x*3,y_start+step_y*3]; ...   % another bulk of points
				%[x_start+step_x,y_start+step_y*3];[x_start+step_x*3,y_start+step_y*2];...
				%[x_start+step_x*2,y_start+step_y*3]...
				%];
				%
			end	
			if (strcmp(boundary_type,'list'))
				boundaries=[[147 118];
				[151 128];
				[166 137 ];
				[170 127 ];
				[167 104 ];
				[151 101 ];
				[179 122 ];
				[179 119 ];
				[149 124 ];
				[154 101 ];
				[171 137 ];
				[155 135 ]];
			end
			if (strcmp(boundary_type,'random'))
				boundaries=floor(rand(n_manual_input_points,2)*200+20);
			end
	 		if (strcmp(boundary_type,'manual'))
	 			%for g=1:1:n_manual_input_points
	 				%
	 				%imshow(img(:,:,1));
					%[x_input, y_input, buttons_data] = ginput([1 10]);
						%%y_input
						%%x_input
						%%buttons_data
				%end
		        		h1=figure;
					imshow(img(:,:,1));
					disp('Select centre');
					[x,y,button]=ginput(1);
                                        if (button==1)
							round_x0=floor(y);
							round_y0=floor(x);
					end
					done=0;
					g=1;
					while (done==0)
       						disp(strcat('Click image for location of point #', num2str(g),'[out of: ',num2str(n_manual_input_points),']'));
						% line([round_y0-(size(img,1)/12)*g,round_y0+(size(img,1)/12)*g|],[round_x0-(size(img,2)/12)*g,round_x0+(size(img,2)/12)*g], 'Color',boundaries_line_colour,'LineWidth',boundaries_line_width,'Marker',boundaries_line_marker,'LineStyle',boundaries_line_type);
						line([round_y0-100,round_y0+100],[round_x0-100,round_x0+100], 'Color',boundaries_line_colour,'LineWidth',boundaries_line_width,'Marker',boundaries_line_marker,'LineStyle',boundaries_line_type);
						line([round_y0+100,round_y0-100],[round_x0-100,round_x0+100], 'Color',boundaries_line_colour,'LineWidth',boundaries_line_width,'Marker',boundaries_line_marker,'LineStyle',boundaries_line_type);
						line([round_y0-0,round_y0+0],[round_x0-100,round_x0+100], 'Color',boundaries_line_colour,'LineWidth',boundaries_line_width,'Marker',boundaries_line_marker,'LineStyle',boundaries_line_type);
						line([round_y0-100,round_y0+100],[round_x0-0,round_x0+0], 'Color',boundaries_line_colour,'LineWidth',boundaries_line_width,'Marker',boundaries_line_marker,'LineStyle',boundaries_line_type);
						line([round_y0-100,round_y0+100],[round_x0-50,round_x0+50], 'Color',boundaries_line_colour,'LineWidth',boundaries_line_width,'Marker',boundaries_line_marker,'LineStyle',boundaries_line_type);
						line([round_y0-50,round_y0+50],[round_x0-100,round_x0+100], 'Color',boundaries_line_colour,'LineWidth',boundaries_line_width,'Marker',boundaries_line_marker,'LineStyle',boundaries_line_type);
						line([round_y0+50,round_y0-50],[round_x0-100,round_x0+100], 'Color',boundaries_line_colour,'LineWidth',boundaries_line_width,'Marker',boundaries_line_marker,'LineStyle',boundaries_line_type);
						line([round_y0-100,round_y0+100],[round_x0+50,round_x0-50], 'Color',boundaries_line_colour,'LineWidth',boundaries_line_width,'Marker',boundaries_line_marker,'LineStyle',boundaries_line_type);
    						[x,y,button]=ginput(1);
    						if (button==1)
        						boundaries(g,1)=floor(y);
        						boundaries(g,2)=floor(x);
        						%Xarray(g)=x;
        						%Yarray(g)=y;
        						g=g+1;
    						if (g==n_manual_input_points+1)
        						done=1;
    						end
					end
	 				% boundaries(:,1)=Xarray';
	 				% boundaries(:,2)=Yarray';
	 			end	
	 				
			end
			if (strcmp(boundary_type,'circle')) % see function cardsim_frame_shortaxis for similar implementation
				point_counter=0;
				for g=1:round_skip_rate:360
					point_counter=point_counter+1; % in Octave one can use point_counter++ and point_counter--, MATLAB does not support it.
         				theta=g*pi/180;
   	 				x=round_x0+round((radius)*(cos(theta)));
    	 				y=round_y0+round((radius)*(sin(theta)));
    	 				% x
    	 				% y
    					boundaries(point_counter,1)=[x];
    					boundaries(point_counter,2)=[y];
    				end
    				
			end
			if (search_along_gradient)  % find better edges to initialise points at
	   		for w=1:1:size(boundaries,1)
	   			%boundaries(w,1)
	   			%boundaries(w,1)+5
	   			%boundaries(w,2)
	   			%a(w,:)=img((boundaries(w,1)-5):1:(boundaries(w,1)+5),(boundaries(w,2)),i)
	   			gradients_x=abs(img(boundaries(w,1)-gradient_search_range:1:boundaries(w,1)+gradient_search_range,boundaries(w,2),i)-img(boundaries(w,1)-gradient_search_range+1:1:boundaries(w,1)+gradient_search_range+1,boundaries(w,2),i));
	   			max_difference_x=findit(gradients_x,1);
   	        		% boundaries(w,1)=boundaries(w,1)-gradient_search_range+max_difference_x(1);   % optionally modify this dimension before looking at the next one
	   			% boundaries(i,1)
	   			% boundaries(i,2)
	   			gradients_y=abs(img(boundaries(w,1), boundaries(w,2)-gradient_search_range:1:boundaries(w,2)+gradient_search_range,i)-img(boundaries(w,1), boundaries(w,2)-gradient_search_range+1:1:boundaries(w,2)+gradient_search_range+1,i));
	   			max_difference_y=findit(gradients_y,1);
   	        		boundaries(w,2)=boundaries(w,2)-gradient_search_range+max_difference_y(1);
   	        		boundaries(w,1)=boundaries(w,1)-gradient_search_range+max_difference_x(1);
	   		end	
	   	end
	end
	% boundaries_x=[;    % deprecated - no more xy separation
        	for l=1:1:size(boundaries,1)
        	% for k=1:1:size(boundaries,2)
        		% debugging
        		% boundaries_y(k,1);
        		% boundaries_y(2,l);
                     	% img(1,3,1)=1;
                     	% boundaries_y(k,1)
                     	% boundaries_y(2,l)
                     	% i
                     	% k
                     	% l
                     	% size(boundaries_y,1)
                     		if (first_image==0)
                     			% old_boundaries
                     	    		new_dot_x=old_boundaries(l,1)+1;
                     	    		new_dot_y=old_boundaries(l,2)+1;
                     	    			% old_boundaries(l,1)-8
					% search_frame_current2=img_previous(((new_dot_x-frame_size):(new_dot_x+frame_size)),((new_dot_y-frame_size):(new_dot_y+frame_size)));
                     	    		search_frame_current=img(((new_dot_x-frame_size):(new_dot_x+frame_size)),((new_dot_y-frame_size):(new_dot_y+frame_size)), i);
                     	    		search_frame_previous=img(((old_boundaries(l,1)-frame_size):(old_boundaries(l,1)+frame_size)),((old_boundaries(l,2)-frame_size):(old_boundaries(l,2)+frame_size)), i-1);
                     	    		% ssearch_frame_previous=img_previous(((old_boundaries(l,1)-frame_size):(old_boundaries(l,1)+frame_size)),((old_boundaries(l,2)-frame_size):(old_boundaries(l,2)+frame_size)), i-1);
                     	    		% search_frame_current2-search_frame_current
                     	    		shuffle_diff(1)=shuffle_transform(search_frame_current, search_frame_previous, shuffle_radius, transform_type);
                     	    		
                     	    		% possible to compress the above into a single line, but then less legible. The following can also be made to be less repetitive and explore a wider range of neighbouring points
                     	    		
                     	    		new_dot_x=old_boundaries(l,1)-1;
                     	    		new_dot_y=old_boundaries(l,2)-1;
                     	    		search_frame_current=img(((new_dot_x-frame_size):(new_dot_x+frame_size)),((new_dot_y-frame_size):(new_dot_y+frame_size)), i);
                     	    		search_frame_previous=img(((old_boundaries(l,1)-frame_size):(old_boundaries(l,1)+frame_size)),((old_boundaries(l,2)-frame_size):(old_boundaries(l,2)+frame_size)), i-1);
                     	    		shuffle_diff(2)=shuffle_transform(search_frame_current, search_frame_previous, shuffle_radius, transform_type);
                     	    		
                     	    		new_dot_x=old_boundaries(l,1)+1;
                     	    		new_dot_y=old_boundaries(l,2)-1;
                     	    		search_frame_current=img(((new_dot_x-frame_size):(new_dot_x+frame_size)),((new_dot_y-frame_size):(new_dot_y+frame_size)), i);
                     	    		search_frame_previous=img(((old_boundaries(l,1)-frame_size):(old_boundaries(l,1)+frame_size)),((old_boundaries(l,2)-frame_size):(old_boundaries(l,2)+frame_size)), i-1);
                     	    		shuffle_diff(3)=shuffle_transform(search_frame_current, search_frame_previous, shuffle_radius, transform_type);
                     	    		
                     	    		new_dot_x=old_boundaries(l,1)-1;
                     	    		new_dot_y=old_boundaries(l,2)+1;
                     	    		search_frame_current=img(((new_dot_x-frame_size):(new_dot_x+frame_size)),((new_dot_y-frame_size):(new_dot_y+frame_size)), i);
                     	    		search_frame_previous=img(((old_boundaries(l,1)-frame_size):(old_boundaries(l,1)+frame_size)),((old_boundaries(l,2)-frame_size):(old_boundaries(l,2)+frame_size)), i-1);
                     	    		shuffle_diff(4)=shuffle_transform(search_frame_current, search_frame_previous, shuffle_radius, transform_type);
                     	    		
                     	    		new_dot_x=old_boundaries(l,1);
                     	    		new_dot_y=old_boundaries(l,2)+1;
                     	    		search_frame_current=img(((new_dot_x-frame_size):(new_dot_x+frame_size)),((new_dot_y-frame_size):(new_dot_y+frame_size)), i);
                     	    		search_frame_previous=img(((old_boundaries(l,1)-frame_size):(old_boundaries(l,1)+frame_size)),((old_boundaries(l,2)-frame_size):(old_boundaries(l,2)+frame_size)), i-1);
                     	    		shuffle_diff(5)=shuffle_transform(search_frame_current, search_frame_previous, shuffle_radius, transform_type);
                     	    		
                     	    		new_dot_x=old_boundaries(l,1)+1;
                     	    		new_dot_y=old_boundaries(l,2);
                     	    		search_frame_current=img(((new_dot_x-frame_size):(new_dot_x+frame_size)),((new_dot_y-frame_size):(new_dot_y+frame_size)), i);
                     	    		search_frame_previous=img(((old_boundaries(l,1)-frame_size):(old_boundaries(l,1)+frame_size)),((old_boundaries(l,2)-frame_size):(old_boundaries(l,2)+frame_size)), i-1);
                     	    		shuffle_diff(6)=shuffle_transform(search_frame_current, search_frame_previous, shuffle_radius, transform_type);
                     	    		
                     	    		new_dot_x=old_boundaries(l,1);
                     	    		new_dot_y=old_boundaries(l,2)-1;
                     	    		search_frame_current=img(((new_dot_x-frame_size):(new_dot_x+frame_size)),((new_dot_y-frame_size):(new_dot_y+frame_size)), i);
                     	    		search_frame_previous=img(((old_boundaries(l,1)-frame_size):(old_boundaries(l,1)+frame_size)),((old_boundaries(l,2)-frame_size):(old_boundaries(l,2)+frame_size)), i-1);
                     	    		shuffle_diff(7)=shuffle_transform(search_frame_current, search_frame_previous, shuffle_radius, transform_type);
                     	    		
                     	    		new_dot_x=old_boundaries(l,1)-1;
                     	    		new_dot_y=old_boundaries(l,2);
                     	    		search_frame_current=img(((new_dot_x-frame_size):(new_dot_x+frame_size)),((new_dot_y-frame_size):(new_dot_y+frame_size)), i);
                     	    		search_frame_previous=img(((old_boundaries(l,1)-frame_size):(old_boundaries(l,1)+frame_size)),((old_boundaries(l,2)-frame_size):(old_boundaries(l,2)+frame_size)), i-1);
                     	    		shuffle_diff(8)=shuffle_transform(search_frame_current, search_frame_previous, shuffle_radius, transform_type);
                     	    		
                     	    		new_dot_x=old_boundaries(l,1);
                     	    		new_dot_y=old_boundaries(l,2);
                     	    		search_frame_current=img(((new_dot_x-frame_size):(new_dot_x+frame_size)),((new_dot_y-frame_size):(new_dot_y+frame_size)), i);
                     	    		search_frame_previous=img(((old_boundaries(l,1)-frame_size):(old_boundaries(l,1)+frame_size)),((old_boundaries(l,2)-frame_size):(old_boundaries(l,2)+frame_size)), i-1);
                     	    		shuffle_diff(9)=shuffle_transform(search_frame_current, search_frame_previous, shuffle_radius, transform_type);
                     	    		
                     	    		% shuffle_diff;
                                        % negative scoring int his case, where low is good, so take minimum to mean maximum benefit
                                        
                     	    		maximum_shuffle=min(shuffle_diff);
                     	    		
                     	    		if (shuffle_diff(1)==maximum_shuffle)
                     	    			new_dot_x=old_boundaries(l,1)+1;
                     	    			new_dot_y=old_boundaries(l,2)+1;
                     	    	                move_direction='se';
                     	    		end
                     	    		if (shuffle_diff(2)==maximum_shuffle)
                     	    			new_dot_x=old_boundaries(l,1)-1;
                     	    			new_dot_y=old_boundaries(l,2)-1;
                     	    	                move_direction='nw';
                     	    		end
                     	    		if (shuffle_diff(3)==maximum_shuffle)
                     	    			new_dot_x=old_boundaries(l,1)+1;
                     	    			new_dot_y=old_boundaries(l,2)-1;
                     	    	                move_direction='sw';
                     	    		end
                     	    		if (shuffle_diff(4)==maximum_shuffle)
                     	    			new_dot_x=old_boundaries(l,1)-1;
                     	    			new_dot_y=old_boundaries(l,2)+1;
                     	    	                move_direction='ne';
                     	    		end
                     	    		if (shuffle_diff(5)==maximum_shuffle)
                     	    			new_dot_x=old_boundaries(l,1);
                     	    			new_dot_y=old_boundaries(l,2)+1;
                     	    	                move_direction='e';
                     	    		end
                     	    		if (shuffle_diff(6)==maximum_shuffle)
                     	    			new_dot_x=old_boundaries(l,1)+1;
                     	    			new_dot_y=old_boundaries(l,2);
                     	    	                move_direction='s';
                     	    		end
                     	    		if (shuffle_diff(7)==maximum_shuffle)
                     	    			new_dot_x=old_boundaries(l,1);
                     	    			new_dot_y=old_boundaries(l,2)-1;
                     	    	                move_direction='w';
                     	    		end
                     	    		if (shuffle_diff(8)==maximum_shuffle)
                     	    			new_dot_x=old_boundaries(l,1)-1;
                     	    			new_dot_y=old_boundaries(l,2);
                     	    	                move_direction='n';
                     	    		end
                     	    		if (shuffle_diff(9)==maximum_shuffle)  % no movement overrides movement with similar effect
                     	    			new_dot_x=old_boundaries(l,1);
                     	    			new_dot_y=old_boundaries(l,2);
                     	    	                move_direction='none';
                     	    		end
                     	    		
                     	    		shuffle_diff;
                     	    		
                     	    		% analyse the directions
                     	    		                
                     	    		if (measure_tangent_component)
                     	    			% check direction relative to centre, (round_x0, round_y0)
                     	    			if (round_x0>new_dot_x)  % point below centre
                     	    				if  (round_y0>new_dot_y)   % point to the left of centre
                     	    					switch lower(move_direction)
   									case {'se'}
      										clockwise=clockwise-2; % in Octave one can use clockwise++ and clockwise--, MATLAB does not support it.
   									case {'nw'}
      										clockwise=clockwise+2;
   									case {'sw'}
      										clockwise;
   									case {'ne'}
      										clockwise;
   									case {'e'}
      										clockwise=clockwise-1;
   									case {'s'}
      										clockwise=clockwise-1;
   									case {'w'}
      										cclockwise=clockwise+1;
   									case {'n'}
      										clockwise=clockwise+1;
   									case {'none'}
      										clockwise;
   									otherwise
      										disp('Unknown direction.')
									end
                     	    				else
                     	    					switch lower(move_direction)
   									case {'se'}
      										clockwise;
   									case {'nw'}
      										clockwise;
   									case {'sw'}
      										clockwise=clockwise+2;
   									case {'ne'}
      										clockwise=clockwise-2;
   									case {'e'}
      										clockwise=clockwise-1;
   									case {'s'}
      										clockwise=clockwise+1;
   									case {'w'}
      										clockwise=clockwise+1;
   									case {'n'}
      										clockwise=clockwise-1;
   									case {'none'}
      										clockwise;
   									otherwise
      										disp('Unknown direction.')
									end
                     	    				end
                     	    			else % point above	
 	                    	    			if (round_y0>new_dot_y)  
                     	    					switch lower(move_direction)
   									case {'se'}
      										clockwise;
   									case {'nw'}
      										clockwise;
   									case {'sw'}
      										clockwise=clockwise-2;
   									case {'ne'}
      										clockwise=clockwise+2;
   									case {'e'}
      										clockwise=clockwise+1;
   									case {'s'}
      										clockwise=clockwise-1;
   									case {'w'}
      										clockwise=clockwise-1;
   									case {'n'}
      										clockwise=clockwise+1;
   									case {'none'}
      										clockwise;
   									otherwise
      										disp('Unknown direction.')
									end
                     	    				else % point to the right
                     	    					switch lower(move_direction)
   									case {'se'}
      										clockwise=clockwise+2;
   									case {'nw'}
      										clockwise=clockwise-2;
   									case {'sw'}
      										clockwise;
   									case {'ne'}
      										clockwise;
   									case {'e'}
      										clockwise=clockwise+1;
   									case {'s'}
      										clockwise=clockwise+1;
   									case {'w'}
      										clockwise=clockwise-1;
   									case {'n'}
      										clockwise=clockwise-1;
   									case {'none'}
      										clockwise;
   									otherwise
      										disp('Unknown direction.')
									end
                     	    				end
                     	    			end
                     	    		end	
                     	    			
                     	    		if (measure_inflation_component)
                     	    			% check direction relative to centre, (round_x0, round_y0)
                     	    			if (round_x0>new_dot_x)  % point below centre
                     	    				if  (round_y0>new_dot_y)   % point to the left of centre
                     	    					switch lower(move_direction)
   									case {'se'}
      										inflation;
   									case {'nw'}
      										inflation;
   									case {'sw'}
      										inflation=inflation+2;
   									case {'ne'}
      										inflation=inflation-2;
   									case {'e'}
      										inflation=inflation-1;
   									case {'s'}
      										inflation=inflation+1;
   									case {'w'}
      										inflation=inflation+1;
   									case {'n'}
      										inflation=inflation-1;
   									case {'none'}
      										inflation;
   									otherwise
      										disp('Unknown direction.')
									end
                     	    				else
                     	    					switch lower(move_direction)
   									case {'se'}
      										inflation=inflation+2;
   									case {'nw'}
      										inflation=inflation-2;
   									case {'sw'}
      										inflation;
   									case {'ne'}
      										inflation;
   									case {'e'}
      										inflation=inflation+1;
   									case {'s'}
      										inflation=inflation+1;
   									case {'w'}
      										inflation=inflation-1;
   									case {'n'}
      										inflation=inflation-1;
   									case {'none'}
      										inflation;
   									otherwise
      										disp('Unknown direction.')
									end
                     	    				end
                     	    			else % point above	
 	                    	    			if (round_y0>new_dot_y)  
                     	    					switch lower(move_direction)
   									case {'se'}
      										inflation=inflation-2;
   									case {'nw'}
      										inflation=inflation+2;
   									case {'sw'}
      										inflation;
   									case {'ne'}
      										inflation;
   									case {'e'}
      										inflation=inflation-1;
   									case {'s'}
      										inflation=inflation-1;
   									case {'w'}
      										inflation=inflation+1;
   									case {'n'}
      										inflation=inflation+1;
   									case {'none'}
      										inflation;
   									otherwise
      										disp('Unknown direction.')
									end
                     	    				else % point to the right
                     	    					switch lower(move_direction)
   									case {'se'}
      										inflation;
   									case {'nw'}
      										inflation;
   									case {'sw'}
      										inflation=inflation-2;
   									case {'ne'}
      										inflation=inflation+2;
   									case {'e'}
      										inflation=inflation+1;
   									case {'s'}
      										inflation=inflation-1;
   									case {'w'}
      										inflation=inflation-1;
   									case {'n'}
      										inflation=inflation+1;
   									case {'none'}
      										inflation;
   									otherwise
      										disp('Unknown direction.')
									end
                     	    				end
                     	    			end
                     	    		end	                     	    			
                     	    			
                     	    		
                     	    		% arrow and colouring starts here
                     	    		
                     	    		if (~strcmp(arrow_type,'none') && ~strcmp(arrow_type,'dot'))
                     	    			% img(new_dot_x,new_dot_y,i)=threshold/255;
                     	    			img(old_boundaries(l,1),old_boundaries(l,2),i)=arrow_color_origin;
                     	    			img(new_dot_x,new_dot_y,i)=arrow_color_second;
                     	    			
                     	    			img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1),(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2),i)=arrow_color_third;
                     	    			img((old_boundaries(l,1)-new_dot_x)*2+old_boundaries(l,1),(old_boundaries(l,2)-new_dot_y)*2+old_boundaries(l,2),i)=arrow_color_origin;
                     	    			img((old_boundaries(l,1)-new_dot_x)+old_boundaries(l,1),(old_boundaries(l,2)-new_dot_y)+old_boundaries(l,2),i)=arrow_color_origin;
                     	    			% img((old_boundaries(l,1)-new_dot_x)*-3+old_boundaries(l,1)+1,(old_boundaries(l,2)-new_dot_y)*-3+old_boundaries(l,2),i)=210;
                     	    			% img((old_boundaries(l,1)-new_dot_x)*-3+old_boundaries(l,1),(old_boundaries(l,2)-new_dot_y)*-3+old_boundaries(l,2-1)+1,i)=210;
                     	    			% img((old_boundaries(l,1)-new_dot_x)*-3+old_boundaries(l,1)-1,(old_boundaries(l,2)-new_dot_y)*-3+old_boundaries(l,2),i)=210;
                     	    			% img((old_boundaries(l,1)-new_dot_x)*-3+old_boundaries(l,1),(old_boundaries(l,2)-new_dot_y)*-3+old_boundaries(l,2)-1,i)=210;
                     	    			% img((old_boundaries(l,1)-new_dot_x)*-4+old_boundaries(l,1),(old_boundaries(l,2)-new_dot_y)*-4+old_boundaries(l,2),i)=210;
                     	    				
                     	    			if ((old_boundaries(l,1)-new_dot_x)*(old_boundaries(l,2)-new_dot_y)==0) % straight arrow:	
                     	    				img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)+1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2),i)=arrow_color_blur;
                     	    				img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)-1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2),i)=arrow_color_blur;
                     	    				img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1),(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)-1,i)=arrow_color_blur;
                     	    				img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1),(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)+1,i)=arrow_color_blur;
                     	    			else % diagonal:
                     	    				img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)+1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)-1,i)=arrow_color_blur;
                     	    				img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)-1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)+1,i)=arrow_color_blur;
                     	    				img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)-1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)-1,i)=arrow_color_blur;
                     	    				img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)+1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)+1,i)=arrow_color_blur;
                     	    			end	
                     	    			img((old_boundaries(l,1)-new_dot_x)*-1+old_boundaries(l,1), ...
                     	    			(old_boundaries(l,2)-new_dot_y)*-1+old_boundaries(l,2),i)=arrow_color_fourth; % draw over
                     	    			
                     	    			if (thick_arrows)
                     	    				if ((old_boundaries(l,1)-new_dot_x)*(old_boundaries(l,2)-new_dot_y)==0) %straight arrow:	
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)+2,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2),i)=arrow_color_blur;
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)-2,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2),i)=arrow_color_blur;
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1),(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)-2,i)=arrow_color_blur;
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1),(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)+2,i)=arrow_color_blur;
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)+1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)-1,i)=arrow_color_blur;
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)-1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)+1,i)=arrow_color_blur;
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)-1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)-1,i)=arrow_color_blur;
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)+1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)+1,i)=arrow_color_blur;
                     	    				else % diagonal:
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)+2,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)-2,i)=arrow_color_blur;
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)-2,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)+2,i)=arrow_color_blur;
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)-2,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)-2,i)=arrow_color_blur;
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)+2,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)+2,i)=arrow_color_blur;
                     	    					% img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)+2,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)+1,i)=arrow_color_blur;
                     	    					% img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)-2,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)-1,i)=arrow_color_blur;
                     	    					% img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)-1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)-2,i)=arrow_color_blur;
                     	    					% img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)+1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)+2,i)=arrow_color_blur;
                     	    					
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1),(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)-1,i)=arrow_color_blur;
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)-1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2),i)=arrow_color_blur;
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1)+1,(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2),i)=arrow_color_blur;
                     	    					img((old_boundaries(l,1)-new_dot_x)*-2+old_boundaries(l,1),(old_boundaries(l,2)-new_dot_y)*-2+old_boundaries(l,2)+1,i)=arrow_color_blur;
                     	    				end	
                     	    			end
                     	    		end
                     	    		
                     	    		if (strcmp(arrow_type,'dot'))
                     	    		  	 img(old_boundaries(l,1),old_boundaries(l,2),i)=230;
                     	    		end
                     	    		
                     	    		
                     	    		old_boundaries(l,1)=new_dot_x;
                     	    		old_boundaries(l,2)=new_dot_y;
                     	    		
                     	    		
                     	    		if (search_along_gradient_every_iteration)  % will override the above
				        	gradients_x=abs(img(old_boundaries(l,1)-gradient_search_range:1:old_boundaries(l,1)+gradient_search_range,old_boundaries(l,2),i)-img(old_boundaries(l,1)-gradient_search_range+1:1:old_boundaries(l,1)+gradient_search_range+1,old_boundaries(l,2),i));
	   					max_difference_x=findit(gradients_x,1);
   	        				old_boundaries(l,1)=old_boundaries(l,1)-gradient_search_range+max_difference_x(1);
	   					gradients_y=abs(img(old_boundaries(l,1), old_boundaries(l,2)-gradient_search_range:1:old_boundaries(l,2)+gradient_search_range,i)-img(old_boundaries(l,1), old_boundaries(l,2)-gradient_search_range+1:1:old_boundaries(l,2)+gradient_search_range+1,i));
	   					max_difference_y=findit(gradients_y,1);
   	        				old_boundaries(l,2)=old_boundaries(l,2)-gradient_search_range+max_difference_y(1);
	                                end
                     	    		
                     		else % first image
                     	    		% img(boundaries(l,1),boundaries(l,2),i)=(255-threshold)/255;
                     	    		img(boundaries(l,1),boundaries(l,2),i)=threshold; % reworked for simplicity, just intensity value
                     	    		set(gcf, 'Position', get(0,'Screensize'));
                     		end
                    % end	
        	end
        	% for l=1:1:size(boundaries_x,1)
        		%for k=1:1:size(boundaries_x,2)
                     	   	% img(boundaries_x(k,2),boundaries_x(l,1),i)=250;
                	%end
        	% end
        	
        	% show visual output
        	
        	if (strcmp(output_type, 'surface'))
			figure;
        		out.hs=surface(img(:,:,i));
			% out.x=x; out.y=y; out.z=z;
			if (using_octave)	
				title(strcat('image #', num2str(i), ':  ', files_list(i,:)))
                     	end
			xlabel('y'); ylabel('x');
		end
        	if (strcmp(output_type, 'surf'))
			figure;
        		out.hs=surf(img(:,:,i));
			if (using_octave)	
				title(strcat('image #', num2str(i), ':  ', files_list(i,:)))
			end
			
                     	xlabel('y'); ylabel('x');
		end
        	if (strcmp(output_type, 'mesh'))
			figure;
        		out.hs=mesh(img(:,:,i));
        		if (using_octave)	
				title(strcat('image #', num2str(i), ':  ', files_list(i,:)))
			end
			xlabel('y'); ylabel('x');
		end
        	if (strcmp(output_type, 'image'))	
			figure;
        		imshow(img(:,:,i))
        		if (triangulate==1)
        			% size(old_boundaries, 1)
        			% size(old_boundaries, 2)
        			for m=1:1:size(old_boundaries, 1)
        				for n=1:1:size(old_boundaries, 1)
        					if (abs(n-m)==1)
        						% old_boundaries(n,2)
        						% old_boundaries(m,2)
							% line([size(img(:,:,i), 1)-old_boundaries(n,1), old_boundaries(m,1)], [size(img(:,:,i), 2)-old_boundaries(n,2), old_boundaries(m,2)]);
							line([old_boundaries(n, 2), old_boundaries(m, 2)],[old_boundaries(n, 1), old_boundaries(m, 1)], 'Color',boundaries_line_colour,'LineWidth',boundaries_line_width,'Marker',boundaries_line_marker,'LineStyle',boundaries_line_type);
						end
					end
				end
				line([old_boundaries(1, 2), old_boundaries(size(old_boundaries, 1), 2)],[old_boundaries(1, 1), old_boundaries(size(old_boundaries, 1), 1)], 'Color',boundaries_line_colour,'LineWidth',boundaries_line_width,'Marker',boundaries_line_marker,'LineStyle',boundaries_line_type);
			end	
			if (using_octave)	
				   title(strcat('image #', num2str(i), ':  ', files_list(i,:)))
			end
			
			xlabel('y'); ylabel('x');
	   		if (draw_circles==1 && ~first_image)
    					% boundaries
    					boundaries(1,3)=0; % adds a third column for sphfit to tolerate 2-D input
	     				[centre,radius]=sphfit(boundaries);
	     				% radius
	     				% centre	
        				if (strcmp(output_type, 'image')) 		% draw cross
	     						line([centre(2), centre(2)], [centre(1)+radius, centre(1)-radius], 'Color',cross_line_colour,'Marker',boundaries_line_marker,'LineStyle',boundaries_line_type)
	     						line([centre(2)+radius, centre(2)-radius], [centre(1), centre(1)], 'Color',cross_line_colour,'Marker',boundaries_line_marker,'LineStyle',boundaries_line_type)
           				end
	    		end
			
			if (save_images)
				if (i<10)
					added_zeroes='000'; % to make files be listed in direct order
				else
					added_zeroes='00';
				end	
				if (using_octave)	
				    	print (strcat(save_directory,'frame-',added_zeroes,num2str(i),'.png'), '-color', '-FHelvetica:12', '-dpng'); % make video frames
				end
			      end
			%old_boundaries(1)
			%old_boundaries(2)
			%boundaries
        		% line(old_boundaries(1), old_boundaries(2));
        		
			% get(0,'Screensize')
			% img_previous=img(:,:,i)
			% plot3(img(:,:,i))
		end
		if (strcmp(output_type, 'none'))
		 	disp('image not shown');
		end
		set(gcf, 'Position', get(0,'Screensize'));  %full screen (messy to watch in multi-head display)
		
       		if (first_image==1) % a fix for the special case of having no prior frame
	   		old_boundaries=boundaries;
		end
        	first_image=0;	
	directions_accumulator(i)=clockwise;
	expansion_accumulator(i)=inflation;
end

% ╔══════════════════════════════════════════════╗
% ║ loadtaggedsequence ║ Dr. R. S. Schestowitz   ║
% ╟──────────────────────────────────────────────╢
% ║ Show some plots    				        ║
% ╚══════════════════════════════════════════════╝

% rather than horizontal/vertical test for rectangle of points, calculate circulation and scaling

if (measure_tangent_component)
	figure;
	plot(directions_accumulator);
	title('Rotation direction (positive is clockwise)')
	xlabel('frame number'); ylabel('accumlator value'); grid('on');
	if (save_images && using_octave)
		print (strcat(save_directory,'directions_accumulator.png'), '-color', '-FHelvetica:12', '-dpng');
	end
end

if (measure_inflation_component)
	figure;
	plot(expansion_accumulator);
	title('Expansion index (higher indicates expansion of circle)')
	xlabel('frame number'); ylabel('accumlator value'); grid('on');
	if (save_images && using_octave)
		print (strcat(save_directory,'expansion_accumulator.png'), '-color', '-FHelvetica:12', '-dpng');
	end
end

% ╔══════════════════════════════════════════════╗
% ║ loadtaggedsequence ║ Dr. R. S. Schestowitz   ║
% ╟──────────────────────────────────────────────╢
% ║ Record results in file				        ║
% ╚══════════════════════════════════════════════╝

% Needs implementing - Dec. 2010 - done

if (output_paramaters_at_end)
	disp('')
	disp('============ Experiment parameters ==============')
	
	disp('Inputs:')
	
	directory
	wildcard
	threshold
	transform_type
	frame_size
	shuffle_radius
	draw_grid
	arrow_type
	verbosity
	output_type
	
	disp('Inline:')
	
	draw_circles
	triangulate
	measure_tangent_component
	clockwise
	move_direction
	
	measure_inflation_component
	inflation
	
	search_along_gradient
	gradient_search_range
	
	boundaries_line_width
	boundaries_line_type
	boundaries_line_marker
	boundaries_line_colour
	
	cross_line_colour
	
	boundary_type
	
	radius
	round_x0
	round_y0
	round_skip_rate
end

if (play_video)
	cd(save_directory);
	system('animate -delay 20 frame*');
end	
